@@ -452,9 +452,11 @@ class FromRattailToRattailExportConsumer(FromRattailToRattailBase):
         self.local_session.close()
 
     def commit_transaction(self):
-        # TODO: should we ever commit local?
-        #self.local_session.commit()
+        self.local_session.commit()
+        self.local_session.close()
+
         self.target_session.commit()
+        self.target_session.close()
 
     def pre_process_changes(self, session, changes):
         if self.runas_username:
