@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -31,7 +31,6 @@ import datetime
 import sqlalchemy as sa
 from sqlalchemy import orm
 
-from rattail.db import api
 from rattail.time import localtime, make_utc
 
 
@@ -54,10 +53,19 @@ def get_lastrun(config, watcher_key, local=False, tzinfo=True, session=None):
     Retrieve the "last run" time for the datasync watcher thread identifed by
     ``watcher_key``.
     """
+    app = config.get_app()
     # the 'last run' value is maintained as UTC
     lastrun_setting = get_lastrun_setting(config, watcher_key)
     timefmt = get_lastrun_timefmt(config)
-    lastrun = api.get_setting(session, lastrun_setting)
+
+    close = False
+    if not session:
+        session = app.make_session()
+        close = True
+    lastrun = app.get_setting(session, lastrun_setting)
+    if close:
+        session.close()
+
     if lastrun:
         lastrun = datetime.datetime.strptime(lastrun, timefmt)
         if local:
