@@ -185,6 +185,18 @@ class new_phone_requested(Email):
         }
 
 
+class postfix_summary(Email):
+    """
+    Sends a summary of Postfix mail activity.
+    """
+    default_subject = "Postfix Summary"
+
+    def sample_data(self, request):
+        return {
+            'output': "(pflogsumm output goes here)",
+        }
+
+
 class ImporterEmail(Email):
     """
     Sent when a "version catch-up" import is performed, which involves changes.
