@@ -29,6 +29,7 @@ from __future__ import unicode_literals, absolute_import
 import os
 import smtplib
 import logging
+import warnings
 from email.charset import Charset
 from email.message import Message
 from email.mime.multipart import MIMEMultipart
@@ -44,7 +45,7 @@ from mako.exceptions import TopLevelLookupException
 from rattail import exceptions
 from rattail.core import UNSPECIFIED
 from rattail.files import resource_path
-from rattail.util import import_module_path
+from rattail.util import import_module_path, load_entry_points
 from rattail.time import localtime, make_utc
 
 
@@ -88,29 +89,119 @@ class EmailHandler(object):
         self.config = config
         self.enum = self.config.get_enum()
 
+    def get_all_emails(self, **kwargs):
+        """
+        Retrieve the complete set of all Email profiles exposed by the
+        app and installed packages.
+        """
+        emails = {}
+        modules = load_entry_points('rattail.emails').values()
+        for module in modules:
+            mod_emails = self.get_emails_from_module(module)
+            emails.update(dict([(email.key or email.__name__, email)
+                                for email in mod_emails]))
+        return emails
+
+    def get_emails_from_module(self, module, **kwargs):
+        """
+        Return all Email profiles from the given Python module.
+        """
+        emails = []
+        for name in dir(module):
+            obj = getattr(module, name)
+            if (isinstance(obj, type) and issubclass(obj, Email)
+                and not obj.abstract and obj is not Email):
+                emails.append(obj)
+        return emails
+
+    def use_entry_points(self):
+        """
+        Returns a boolean indicating if the app should get its list of
+        available emails from the setuptools entry points, instead of
+        the (legacy) ``[rattail.mail] emails`` config setting.
+        """
+        return self.config.getbool('rattail.mail', 'emails.use_entry_points',
+                                   default=False)
+
+    def get_available_emails(self, **kwargs):
+        """
+        Retrieve the set of "available" Email profiles, i.e. those
+        which should be exposed to the app users.
+        """
+        # TODO: maybe deprecate this at some point?
+        if not self.use_entry_points():
+            # nb. this is the legacy behavior
+            return dict([(email.key or email.__name__, email)
+                         for email in self.iter_emails(warn=False)])
+
+        # this is the future
+        emails = self.get_all_emails()
+        for key in list(emails):
+            if not self.email_is_available(key):
+                emails.pop(key)
+        return emails
+
+    def iter_emails(self, warn=True):
+        """
+        Iterate over all available email types.
+        """
+        if warn:
+            warnings.warn("`iter_emails()` method is deprecated!  please "
+                          "use `get_available_emails()` instead",
+                          DeprecationWarning, stacklevel=2)
+
+        for module in self.config.getlist('rattail.mail', 'emails', default=['rattail.emails']):
+            module = import_module_path(module)
+            for email in self.get_emails_from_module(module):
+                yield email
+
     def get_email(self, key, fallback_key=None, **kwargs):
         """
         Return an email instance of the given type.
         """
-        for email in self.iter_emails():
-            if email.key == key or email.__name__ == key:
-                return email(self.config, key, fallback_key, **kwargs)
-        return Email(self.config, key, fallback_key, **kwargs)
+        factory = Email
+        emails = self.get_all_emails()
+        if key in emails:
+            factory = emails[key]
+        return factory(self.config, key, fallback_key, **kwargs)
 
-    def iter_emails(self):
+    def email_is_available(self, email):
         """
-        Iterate over all available email types.
+        Returns boolean indicating if the Email profile for the given
+        key is "available" and should be exposed to users.
         """
-        for module in self.config.getlist('rattail.mail', 'emails', default=['rattail.emails']):
-            module = import_module_path(module)
-            for name in dir(module):
-                obj = getattr(module, name)
-                if (isinstance(obj, type) and issubclass(obj, Email)
-                    and not obj.abstract and obj is not Email):
-                    yield obj
+        return not self.email_is_hidden(email)
+
+    def email_is_hidden(self, email):
+        """
+        Returns boolean indicating if the given Email profile is
+        "hidden" and should not be exposed to users.
+
+        :param email: Either an :class:`Email` instance, or the
+           :attr:`~Email.key` value for one.
+        """
+        key = getattr(email, 'key', email)
+        hidden = self.config.getbool('rattail.mail',
+                                     '{}.hidden'.format(key),
+                                     default=False)
+        return hidden
+
+    def email_is_enabled(self, email):
+        """
+        Returns boolean indicating if the given Email profile is
+        "enabled" - that is, emails of its type should be sent as
+        opposed to being suppressed.
+
+        :param email: An :class:`Email` instance.
+        """
+        return email.get_enabled()
 
     def get_enabled(self, email):
-        return email.get_enabled()
+        warnings.warn("`get_enabled()` method is deprecated!  please "
+                      "use `email_is_enabled()` instead",
+                      DeprecationWarning, stacklevel=2)
+
+        return self.email_is_enabled(email)
 
     def send_message(self, email, data, **kwargs):
         msg = self.make_message(email, data, **kwargs)
@@ -241,8 +332,12 @@ def get_email_handler(config):
     """
     Creates and returns the configured mail handler.
     """
-    # TODO: should let config override which handler we use
-    return EmailHandler(config)
+    warnings.warn("`get_email_handler()` is deprecated!  please "
+                  "use `AppHandler.get_email_handler()` instead",
+                  DeprecationWarning, stacklevel=2)
+
+    app = config.get_app()
+    return app.get_email_handler()
 
 
 class Email(object):
